import numpy as np
import matplotlib.pyplot as plt
from scipy.io import wavfile

def plot_wav_with_timestamps(wav_path, events_path, event_id="2"):
    # 1. Read the WAV file
    samplerate, data = wavfile.read(wav_path)
    
    # If stereo, select only one channel (e.g., left channel)
    # if data.ndim == 2:
    #     data = data[:, 0]

    # Create time axis in seconds
    duration = len(data) / samplerate
    time_axis = np.linspace(0, duration, num=len(data))

    # 2. Read the events file
    events = read_events(events_path,event_id)
    if not events:
        print(f"No events found with ID={event_id}.")

    # add the 10 second offset for each task epoch
    events_to_plot = [events[0] + 10 * i for i in range(6)]

    # 2) Find indices where data crosses threshold
    threshold = 30000
    mask = data > threshold

    # 3. Plot masked audio waveform
    # mask = (time_axis >= events_to_plot[0]) & (time_axis <= events_to_plot[1])

    # 3. Plot the audio waveform
    plt.figure(figsize=(10, 4))
    plt.plot(time_axis, data, label='EMG')

    # plt.plot(time_axis[~mask], data[~mask], 
    #          color='blue', label='Rest')
    # plt.plot(time_axis[mask], data[mask], 
    #          color='red', label='Contraction')

    plt.scatter(time_axis[mask], np.ones(np.sum(mask)) * threshold, color="red", marker="o", label=f">{threshold}")
    plt.axhline(y=threshold, color="gray", linestyle="--", alpha=0.7)


    plt.xlabel('Time (s)')
    plt.ylabel('Amplitude')
    plt.title(wav_path)

    # 4. Plot a vertical line for each event
    for t in events_to_plot:
        plt.axvline(x=t, color='r', linestyle='--', alpha=0.8)
    
    # Optional: Set plot limits so you can see the lines
    # (especially if the timestamps are near or beyond waveform duration)
    # plt.xlim([0, max(duration, max(timestamps) + 5)])

    plt.legend()
    plt.tight_layout()
    plt.show()

def read_events(filename, marker_id_to_return):
    events = []

    with open(filename, 'r') as f:
        for line in f:
            line = line.strip()
            
            # Skip empty lines or comment lines (lines starting with "#")
            if not line or line.startswith('#'):
                continue
            
            # Each valid line should be "id, time_in_seconds"
            parts = line.split(',')
            if len(parts) < 2:
                continue  # skip malformed lines

            event_id = parts[0].strip()
            try:
                time_in_seconds = float(parts[1].strip())
            except ValueError:
                # If we can't parse the time as float, skip the line
                continue

            # Pick only the timestamps whose ID is "2"
            if event_id == marker_id_to_return:
                events.append(time_in_seconds)

    return events

if __name__ == "__main__":
    subject="SM"
    wav_file_path = fr"data\on_off_10sec\on_off_10s_250110_{subject}.wav"
    events_file = fr"data\on_off_10sec\on_off_10s_250110_{subject}_events.txt" 
    plot_wav_with_timestamps(wav_file_path, events_file)